# Copyright (C) 2006-2011 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

"""
@copyright: Copyright (C) 2008, 2010 Oracle.  All rights reserved.

@license:   This program is free software; you can redistribute it and/or
            modify it under the terms of the GNU General Public
            License as published by the Free Software Foundation, version 2.

            This program is distributed in the hope that it will be useful,
            but WITHOUT ANY WARRANTY; without even the implied warranty of
            MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
            General Public License for more details.

@summary:   OS specific functions for the Oracle Generic NFS plugin.

"""

import gettext
import subprocess

from OSCPlugin import IPlugin, OperationFailedEx, CloneFailedEx


_MOUNT_OPTIONS = ['nosharecache', 'soft', 'fg', 'retry=1']
_MOUNT_OPTIONS_V3 = _MOUNT_OPTIONS + ['udp', 'vers=3']
_MOUNT_OPTIONS_V4 = _MOUNT_OPTIONS

_MOUNT_TYPES = [['nfs', _MOUNT_OPTIONS_V3],
                ['nfs4', _MOUNT_OPTIONS_V4]]

_ = gettext.gettext


def run_cmd(cmd):

    IPlugin.logger.debug('cmd = %s', ' '.join(cmd))

    cmd_p = subprocess.Popen(cmd,
                             stdout = subprocess.PIPE,
                             stderr = subprocess.PIPE,
                             close_fds=True)
    (p_out, p_err) = cmd_p.communicate()

    return (cmd_p.returncode, str(p_out), str(p_err))


def umount(mountpoint):

    (status, p_out, p_err) = run_cmd(['/bin/umount', mountpoint])

    if status != 0:
        err = _('Unable to umount filesystem %s: stdout: %s, stderr: %s') % \
                (mountpoint, p_out, p_err)
        IPlugin.logger.error(err)
        cmd = ['lsof', mountpoint]
        (status, p_out, p_err) = run_cmd(cmd)
        dbg = _('cmd: %s, status: %s, stdout: %s, stderr: %s') % \
                (cmd, status, p_out, p_err)
        IPlugin.logger.debug(dbg)
        cmd = ['dmsetup', 'table', '--target', 'nfs']
        (status, p_out, p_err) = run_cmd(cmd)
        dbg = _('cmd: %s, status: %s, stdout: %s, stderr: %s') % \
                (cmd, status, p_out, p_err)
        IPlugin.logger.debug(dbg)
        raise OperationFailedEx(err)


def mount(resource, mountpoint, options=None):

    if options is not None:
        mount_types = [['nfs', options]]
    else:
        mount_types = _MOUNT_TYPES

    for (fstype, mntopt) in mount_types:

        cmd = ['/bin/mount', '-t', fstype]

        if mntopt:
            cmd += ['-o', ','.join(mntopt)]
            cmd += ['-v']

        cmd += [resource, mountpoint]

        (status, p_out, p_err) = run_cmd(cmd)

        if status == 0:
            break

        if p_err.find ('version mismatch') < 0:
            break


    if status != 0:
        error = _('Unable to mount "%s" on "%s": %s') % (resource,
                                                         mountpoint, p_err)
        IPlugin.logger.error(error)
        raise OperationFailedEx(error)


def clone_file(src, dst):

    (status, p_out, p_err) = run_cmd(['/bin/dd', 'if=%s' % src, 'of=%s' % dst,
                                  'bs=4096k', 'oflag=direct'])

    if status != 0:
        error = _('Unable to clone file "%s" to "%s": %s') % (src, dst,
                                                              p_err)
        IPlugin.logger.error(error)
        raise CloneFailedEx(error)


def get_export(host):

    (status, p_out, p_err) = run_cmd(['/usr/sbin/showmount', '--export', host])

    if status != 0:
        error = _('Unable to access file system at %s: %s' % (host, p_err))
        IPlugin.logger.error(error)
        raise OperationFailedEx(error)

    exports = []
    export_items = p_out.splitlines()
    export_items = export_items[1:]

    for export in export_items:
        exports.append(export.split()[0])

    return exports


def get_mounts():

    mounts = {}
    proc_mounts = open("/proc/mounts", "r")
    try:
        for line in proc_mounts:
            mnt = _proc_mounts_to_mount(line)
            if not mnt['fstype'].startswith('nfs') or mnt['fstype'] == 'nfsd':
                continue
            server, export = mnt['device'].split(":")
            mnt['server'] = server
            mnt['export'] = export
            if mounts.has_key(mnt['mountpoint']):
                mounts[mnt['mountpoint']].append(mnt)
            else:
                mounts[mnt['mountpoint']] = [mnt]
    finally:
        proc_mounts.close()

    return mounts


def get_mount_info(device):

    try:
        proc_mounts = open("/proc/mounts", "r")
        for line in proc_mounts:
            mnt = _proc_mounts_to_mount(line)
            mnt['device'] = mnt['device'].rstrip('/')
            if mnt['fstype'].startswith('nfs') and mnt['device'] == device:
                server, export = mnt['device'].split(":")
                mnt['server'] = server
                mnt['export'] = export
                proc_mounts.close()
                return mnt

    except IOError, ioe:
        proc_mounts.close()
        raise

    proc_mounts.close()
    return {}


def get_status():

    fsys = file("/proc/filesystems", "r")
    try:
        for line in fsys:
            if line.find('nfs') >= 0:
                return "loaded"
        return "not loaded"
    finally:
        fsys.close()


def _proc_mounts_to_mount(line):
    data = line.split()
    return {
        'device': data[0].decode('string_escape'),
        'mountpoint': data[1].decode('string_escape'),
        'fstype': data[2],
        'options': data[3].decode('string_escape')
    }
